`define MUL_5  //MUL_5 / MUL_2
module top(
    clk,
    rstn,
    din,
	in_valid,
	gainwe,
	gainset,
	gain,
	in_ready,
    out_valid,
    dout
	 );
    parameter BAND_SIZES=16;
	parameter GAIN_NUM=3;
	parameter COEFF_WIDTH=20;
	parameter DIN_WIDTH=19;
	localparam NUM_DATA = 500;
	localparam CLK_PERIOD = 50.0;
    localparam DOUT_WIDTH = DIN_WIDTH + 3;
    input clk, rstn;
	output in_valid;
	output reg [DIN_WIDTH-1:0] din;
	output reg [2:0] gain;
    output signed [DOUT_WIDTH-1:0] dout;
	wire ce;
	output reg gainwe, gainset, in_ready, out_valid;

	reg [DIN_WIDTH-1:0] din_data [0:NUM_DATA-1];
    reg [2:0] gain_data [0:BAND_SIZES-1];
	reg [$clog2(NUM_DATA+1):0] i_data;
//**********************************************************************
// --- Define Variable
//**********************************************************************	
	reg [19:0] addr;
	reg [4:0]  cnt_cycle;
    reg [2:0]  cnt;
	reg pstart;
//**********************************************************************
// --- Initial Block
//**********************************************************************
    Equalizer_Top Equalizer_inst(
        .clk(clk),                 // 时钟信号
        .rstn(rstn),               // 复位信号（低电平有效）
		.ce(ce),                   // 时钟使能信号
        .in_valid(in_valid),       // 高电平时表示此时输入数据有效
		.din(din),                 // 输入信号
		.gainwe(gainwe),           // 载入gain时置为高电平
		.gainset(gainset),         // 载入gain完成后下一个时钟周期置为高电平，表示gain载入完成
		.gain(gain),               // 输入gain信号编号
		.in_ready(in_ready),       // 可输入指示信号，高电平时表示数据可以输入
		.out_valid(out_valid),     // 输出有效信号，高电平时表示此时输出信号为有效值
		.dout(dout)                // 输出信号
    );

	assign in_valid  = (cnt_cycle == 0&&!gainwe) ? 1'b1 : 1'b0;
	assign ce = 1'b1;
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			i_data <= BAND_SIZES+1;
		end
		else if(i_data<=BAND_SIZES)begin
			i_data <= i_data + 1'b1;
		end
        else if(addr==5 && cnt==0)begin
			i_data <= 0;
		end
	end

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			cnt <= 0;
		end
		else if(i_data == 0)begin
			cnt <= cnt +1'b1;
		end
	end	
	// always @(posedge clk or negedge rstn) begin
		// if(!rstn) begin
			// ce <= 0;
		// end
		// else if(i_data <= 100) begin
			// ce <= 1'b1;
		// end
		// else begin
			// ce <= 1'b0;
		// end
	// end
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			gainwe <= 0;
		end
		else if(i_data<BAND_SIZES) begin
			gainwe <= 1'b1;
		end
		else begin
			gainwe <= 1'b0;
		end
	end
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			gainset <= 0;
		end
		else if(i_data==BAND_SIZES) begin
			gainset <= 1'b1;
		end
		else begin
			gainset <= 1'b0;
		end
	end

    always @(posedge clk or negedge rstn) begin
        if(!rstn) cnt_cycle <= BAND_SIZES-1;
        else if(ce) begin
			if(cnt_cycle == BAND_SIZES-1 && in_ready ) cnt_cycle <= 0;
            else if(cnt_cycle == BAND_SIZES-1) cnt_cycle <= cnt_cycle;
            else cnt_cycle <= cnt_cycle + 1'b1;
        end
    end
	
    always @(posedge clk or negedge rstn) begin
        if(!rstn) addr <= 0;
        else if(ce) begin
			if(gainwe) addr <= 0;
			else if(pstart && in_ready) begin
				if(cnt_cycle == BAND_SIZES-1) addr <= addr + 1;
			end
        end
    end
	
 always @(posedge clk or negedge rstn) begin
        if(!rstn) begin
			din <= 0;
		end
        else if(ce && i_data==BAND_SIZES+1) begin
			if(cnt_cycle == BAND_SIZES-1 && addr == NUM_DATA-1) begin
				din <= 0;
			end
            else if((cnt_cycle == BAND_SIZES-1)&& in_ready) begin
				din <= din_data[addr];
			end
        end
    end
	
	always @(posedge clk or negedge rstn) begin
        if(!rstn) begin
			gain <= 0;
		end
        else if(ce&&i_data<BAND_SIZES) begin
			gain <= gain_data[i_data];
        end
    end

	always @(posedge clk or negedge rstn) begin
        if(!rstn) pstart <= 0;
        else if(ce) begin
            if(cnt_cycle == BAND_SIZES-1) pstart <= 1'b1;
        end
    end
	
    initial begin
		$readmemb("din.dat",din_data);
		$readmemb("gain.dat",gain_data);
    end
endmodule
